@@ -15,8 +15,18 @@ class WLANInterface(AbstractInterface):
   wifi_module_path = '/org/freedesktop/Hal/Manager'
   wifi_module_iface = 'org.freedesktop.Hal.Manager'
   wifi_module_signal = None
-  wifi_turn_on_cmd = ['sudo /usr/bin/advifsw_wlan_on.sh']
-  wifi_turn_off_cmd = ['sudo /usr/bin/advifsw_wlan_off.sh']
+  wifi_cmd = {'up': 'sudo /usr/bin/advifsw_wlan_up.sh',
+              'down': 'sudo /usr/bin/advifsw_wlan_down.sh',
+              'unload': 'sudo /usr/bin/advifsw_wlan_unload.sh',
+              'load0': 'sudo /usr/bin/advifsw_wlan_load0.sh',
+              'load1': 'sudo /usr/bin/advifsw_wlan_load1.sh',
+              'get_driver': "lsmod | grep '^wl12\([x]\{2\}\|51\)\ ' | sed -n s/\ .*//p",
+              'check_be': 'if [ -d /lib/modules/`uname -r` ]; then echo 1; fi'
+             }
+  # off: down > unload
+  wifi_turn_off_cmd = [wifi_cmd['down'], wifi_cmd['unload']]
+  wifi_drivers = ['wl12xx', 'wl1251']
+  
   wifi_icd2_path = '/com/nokia/icd2'
   wifi_icd2_iface = 'com.nokia.icd2'
   wifi_icd2_state_signal = 'state_sig'
@@ -27,7 +37,7 @@ class WLANInterface(AbstractInterface):
   def pre_init(self):
     self.__strength_timeout = None
     self.__autooff = None
-    self.has_be = get_shell_command_output('ls /lib/modules/current/*.ko | grep -m 1 wl1251') != ''
+    self.has_be = get_shell_command_output(self.wifi_cmd['check_be']) != ''
     self.state = {'name': 'WLAN',
                   'dbus': {'on': {'name': self.wifi_module_signal, 
                                   'iface': self.wifi_module_iface, 
@@ -57,10 +67,10 @@ class WLANInterface(AbstractInterface):
                   'banner': (True, True, True, True, True),
                   'value': InterfaceState.off,
                   'strength': 0,
-                  'modes': ('wlan-stock',),
+                  'modes': ['wlan-stock'],
                   'mode': 0}
     if self.has_be:
-      self.state['modes'] = ('wlan-stock', 'wlan-bleeding-edge')
+      self.state['modes'] = ['wlan-stock', 'wlan-bleeding-edge']
     self.interfaces = {'ICD': [self.wifi_icd2_iface, self.wifi_icd2_path, 
                                self.wifi_icd2_iface, '/usr/sbin/icd2', False],
                        'ICD_UI': [self.wifi_icdui_iface, self.wifi_icdui_path, 
@@ -72,8 +82,10 @@ class WLANInterface(AbstractInterface):
 
   def process_signal_on(self, *args, **kwargs):
     result = None
+    mode = None
     if args[0] == '/org/freedesktop/Hal/devices/platform_wl12xx':
       if kwargs['member'] == 'DeviceAdded':
+        mode = self.__get_mode(self.__get_driver())
         result = InterfaceState.on
         if self.get_config_value('show-connection-dialog'):
           #self.scan_request()
@@ -83,7 +95,7 @@ class WLANInterface(AbstractInterface):
         result = InterfaceState.off
         self.__cancel_autooff()
         self.__cancel_get_strength()
-    return result, None, None
+    return result, mode, None
 
   def process_signal_connected(self, *args):
     result = None
@@ -118,20 +130,21 @@ class WLANInterface(AbstractInterface):
     return None, None, min(int(args[7]), 5)
 
   def get_status(self):
-    loaded = get_shell_command_output('lsmod | grep -m 1 wl12xx') != ''
-    if loaded:
+    driver = self.__get_driver()
+    mode = self.__get_mode(driver)
+    if driver == '':
+      return InterfaceState.off, mode, None
+    else:
       try:
         self.get_interface('ICD').state_req()
       except:
         self.reinit_interface('ICD')
-      return InterfaceState.on, 0, 0
-    else:
-      return InterfaceState.off, 0, 0
+      return InterfaceState.on, mode, 0
 
   def turn_on(self):
     if self.state['value'] == InterfaceState.off:
       #run_async(self.run_commands, (self.wifi_turn_on_cmd,))
-      gobject.idle_add(self.run_commands, self.wifi_turn_on_cmd)
+      gobject.idle_add(self.run_commands, [self.wifi_cmd['load%s' % self.state['mode']], self.wifi_cmd['up']])
     elif self.state['value'] == InterfaceState.connected:
       try:
         self.get_interface('ICD').disconnect_req(dbus.UInt32(0x8000))
@@ -181,9 +194,31 @@ class WLANInterface(AbstractInterface):
       self.__strength_timeout = None
 
   def turn_mode(self, mode):
+    mode = min(mode, len(self.state['modes']) - 1)
+    if self.state['mode'] != mode:
+      if self.state['value'] == InterfaceState.off:
+        pass
+      else:
+        # down > unload > load%mode% > up
+        gobject.idle_add(self.run_commands, [self.wifi_cmd['down'], self.wifi_cmd['load%s' % mode], self.wifi_cmd['up']])
     self.switched()
 
   def __cancel_autooff(self):
     if self.__autooff:
       gobject.source_remove(self.__autooff)
       self.__autooff = None
+
+  def __get_driver(self):
+    return get_shell_command_output(self.wifi_cmd['get_driver']).replace('\n', '')
+
+  def __get_mode(self, driver):
+    config = 0
+    if self.has_be:
+      if driver == '':
+        try:
+          config = self.state['modes'].index(self.get_config_value('mode-on-startup'))
+        except ValueError:
+          self.set_config_value('mode-on-startup', self.state['modes'][config], True)
+      else:
+        config = self.wifi_drivers.index(driver)
+    return config
